#include <stdio.h>
#include <dlfcn.h>

#include "magic_code.h"
#include "open_gui.h"

void *lib_handle;
bool gui_trigger_safe = false;

int open_gui(int argc, char **argv, bool is_debug, int (*start_vpn)(), int (*stop_vpn)(), int (*restart_conf_vpn)(bool), int (*refresh_vpn)()) {
  void (*fn)(int*, int*);
  int (*gui)(int argc, char** argv, bool is_debug, int (*start_vpn)(), int (*stop_vpn)(), int (*restart_conf_vpn)(bool), int (*refresh_vpn)());
  int a = 0;
  int b = 0;
  char *error;
  char *p;

  if(is_debug) p = "./libmelonvpngui.so";
  else p = "/usr/lib/melonvpn/libmelonvpngui.so";

  lib_handle = dlopen(p, RTLD_LAZY);
  if (!lib_handle) {
    fprintf(stderr, "Failed to open Melon VPN gui\n");
    fprintf(stderr, "Try installing the package 'melonvpngui'\n");
    fprintf(stderr, "%s\n", dlerror());
    return 1;
  }

  fn = dlsym(lib_handle, "get_magic_code");
  if ((error = dlerror()) != NULL) {
    fprintf(stderr, "Failed to open Melon VPN gui\n");
    fprintf(stderr, "The function 'get_magic_code' couldn't be found\n");
    fprintf(stderr, "Check the package 'melonvpngui' is installed correctly\n");
    fprintf(stderr, "%s\n", error);
    dlclose(lib_handle);
    return 1;
  }

  (*fn)(&a,&b);

  if(a==MAGIC_CODE_A && b==MAGIC_CODE_B) {
    gui = dlsym(lib_handle, "run_gui");
    if((error = dlerror()) != NULL) {
      fprintf(stderr, "Failed to open Melon VPN gui\n");
      fprintf(stderr, "The function 'run_gui' couldn't be found\n");
      fprintf(stderr, "Check the package 'melonvpngui' is installed correctly\n");
      fprintf(stderr, "%s\n", error);
      dlclose(lib_handle);
      return 1;
    }

    gui_trigger_safe = true;

    int fake_argc = 1;
    char *fake_argv[1] = {0};
    fake_argv[0] = argv[0];

    (*gui)(fake_argc, fake_argv, is_debug, start_vpn, stop_vpn, restart_conf_vpn, refresh_vpn);
  }

  gui_trigger_safe = false;

  dlclose(lib_handle);
  return 0;
}

int gui_trigger_state(States state) {
  if(!gui_trigger_safe) return 100;

  char *error;
  void (*trigger)(States);
  trigger = dlsym(lib_handle, "mod_trigger_state");

  if((error = dlerror()) != NULL) {
    fprintf(stderr, "Failed to run 'mod_trigger_state'\n");
    fprintf(stderr, "%s\n", error);
    return 1;
  }

  (*trigger)(state);
  return 0;
}

int gui_trigger_isrestart(bool isrestart) {
  if(!gui_trigger_safe) return 100;

  char *error;
  void (*trigger)(bool);
  trigger = dlsym(lib_handle, "mod_trigger_isrestart");

  if((error = dlerror()) != NULL) {
    fprintf(stderr, "Failed to run 'mod_trigger_isrestart'\n");
    fprintf(stderr, "%s\n", error);
    return 1;
  }

  (*trigger)(isrestart);
  return 0;
}

int gui_trigger_peers(char *peers) {
  if(!gui_trigger_safe) return 100;

  char *error;
  void (*trigger)(char*);
  trigger = dlsym(lib_handle, "mod_trigger_peers");

  if((error = dlerror()) != NULL) {
    fprintf(stderr, "Failed to run 'mod_trigger_peers'\n");
    fprintf(stderr, "%s\n", error);
    return 1;
  }

  (*trigger)(peers);
  return 0;
}
