#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libappindicator/app-indicator.h>
#include <stdbool.h>

#include "states.h"
#include "statusicon.h"

GtkWidget *mainwindow;
AppIndicator *indicator;
GtkWidget *indicator_menu;
States status_icon_currentstate;

bool isquitcallback = false;
bool isconnectcallback = false;
bool isdisconnectcallback = false;
int (*quit_callback)();
int (*connect_callback)();
int (*disconnect_callback)();

static void activate_action (GtkAction *action);

static GtkActionEntry entries[] = {
  {"FileMenu",NULL,"_File"},
  {"Connect","vpn-connect","_Connect","<control>C","Connect to VPN",G_CALLBACK(activate_action)},
  {"Disconnect","vpn-disconnect","_Disconnect","<control>D","Disconnect from VPN",G_CALLBACK(activate_action)},
  {"Open","window-open","_Open","<control>O","Open the window",G_CALLBACK(activate_action)},
  {"Close","window-close","_Close","<control>W","Close the window",G_CALLBACK(activate_action)},
  {"Quit","application-exit","_Quit","<control>Q","Exit the application",G_CALLBACK(activate_action)},
};
static guint n_entries = G_N_ELEMENTS (entries);

static const gchar *ui_info =
"<ui>"
"  <popup name='IndicatorPopup'>"
"    <menuitem action='Connect' />"
"    <menuitem action='Disconnect' />"
"    <separator/>"
"    <menuitem action='Open' />"
"    <menuitem action='Close' />"
"    <menuitem action='Quit' />"
"  </popup>"
"</ui>";

static void activate_action(GtkAction *action) {
  const gchar *name = gtk_action_get_name(action);
  if(strcmp(name,"Quit") == 0) {
    // A custom callback is used as closing the window triggers gtk_widget_hide
    if(isquitcallback) (*quit_callback)();
    return;
  } else if(strcmp(name,"Close") == 0) {
    // Hide the main window
    if(mainwindow != NULL) gtk_widget_hide(GTK_WIDGET(mainwindow));
    return;
  } else if(strcmp(name,"Open") == 0) {
    // Open the main window
    if(mainwindow != NULL) gtk_widget_show(GTK_WIDGET(mainwindow));
    return;
  } else if(strcmp(name,"Connect") == 0) {
    // Trigger connect to VPN callback
    if(isconnectcallback) (*connect_callback)();
    return;
  } else if(strcmp(name,"Disconnect") == 0) {
    // Trigger disconnect from VPN callback
    if(isdisconnectcallback) (*disconnect_callback)();
    return;
  }

  GtkWidget *dialog;
  dialog = gtk_message_dialog_new(NULL,GTK_DIALOG_DESTROY_WITH_PARENT,GTK_MESSAGE_INFO,GTK_BUTTONS_CLOSE,"You activated action: \"%s\"",name);
  g_signal_connect(dialog,"response",G_CALLBACK(gtk_widget_destroy),NULL);
  gtk_widget_show(dialog);
}

void status_icon_create(GtkWidget *window, char *statusoff, char *statuson) {
  if(indicator == NULL) {
    mainwindow = window;
    GtkUIManager *uim;
    GError *error = NULL;
    GtkActionGroup *action_group;

    action_group = gtk_action_group_new("AppActions");
    gtk_action_group_add_actions(action_group,entries,n_entries,window);

    uim = gtk_ui_manager_new();
    g_object_set_data_full(G_OBJECT(window),"ui-manager",uim,g_object_unref);
    gtk_ui_manager_insert_action_group(uim,action_group,0);
    gtk_window_add_accel_group(GTK_WINDOW(window),gtk_ui_manager_get_accel_group(uim));

    if (!gtk_ui_manager_add_ui_from_string(uim,ui_info,-1,&error)) {
      g_message("Failed to build menus: %s\n",error->message);
      g_error_free(error);
      error = NULL;
    }
    
    indicator = app_indicator_new("melonvpn",statusoff,APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
    indicator_menu = gtk_ui_manager_get_widget(uim,"/ui/IndicatorPopup");
    app_indicator_set_status(indicator,APP_INDICATOR_STATUS_PASSIVE);
    app_indicator_set_attention_icon(indicator,statuson);
    app_indicator_set_menu(indicator,GTK_MENU(indicator_menu));
  }
}

void status_icon_set_quit_callback(int (*func)()) {
  quit_callback = func;
  isquitcallback = true;
}

void status_icon_set_connect_callback(int (*func)()) {
  connect_callback = func;
  isconnectcallback = true;
}

void status_icon_set_disconnect_callback(int (*func)()) {
  disconnect_callback = func;
  isdisconnectcallback = true;
}

void status_icon_set_state(AppIndicatorStatus status) {
  app_indicator_set_status(indicator,status);
  app_indicator_set_menu(indicator,GTK_MENU(indicator_menu));
}
