#include "unix_sockets.h"

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

int unix_socket_server_start(char *sock_path) {
  struct sockaddr_un addr;

  // Create a new server socket with domain: AF_UNIX, type: SOCK_STREAM, protocol: 0
  int sfd = socket(AF_UNIX, SOCK_STREAM, 0);

  // Make sure socket's file descriptor is legit.
  if (sfd == -1) {
    fprintf(stderr,"ERROR: socket is not legit\n");
    return -1;
  }

  // Make sure the address we're planning to use isn't too long.
  if (strlen(sock_path) > sizeof(addr.sun_path) - 1) {
    fprintf(stderr,"ERROR: Server socket path too long: %s\n",sock_path);
    return -1;
  }

  // Delete any file that already exists at the address. Make sure the deletion
  // succeeds. If the error is just that the file/directory doesn't exist, it's fine.
  if (remove(sock_path) == -1 && errno != ENOENT) {
    fprintf(stderr,"ERROR: Failed to remove old socket file: %s\n",sock_path);
    return -1;
  }

  // Zero out the address, and set family and path.
  memset(&addr, 0, sizeof(struct sockaddr_un));
  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, sock_path, sizeof(addr.sun_path) - 1);

  // Bind the socket to the address. Note that we're binding the server socket
  // to a well-known address so that clients know where to connect.
  if (bind(sfd, (struct sockaddr *) &addr, sizeof(struct sockaddr_un)) == -1) {
    fprintf(stderr,"ERROR: Failed to bind to socket address\n");
    return -1;
  }

  // The listen call marks the socket as *passive*. The socket will subsequently
  // be used to accept connections from *active* sockets.
  // listen cannot be called on a connected socket (a socket on which a connect()
  // has been succesfully performed or a socket returned by a call to accept()).
  if (listen(sfd, BACKLOG) == -1) {
    fprintf(stderr,"ERROR: Failed to listen to socket\n");
    return -1;
  }

  return sfd;
}
